#! /usr/bin/env python3
import sys
import os
import json
import logging
import re

# If you want to debug the auto completion framework, then make below as logging.DEBUG
logging.basicConfig(level=logging.INFO)  

class Loader:
    def get_flavor(self):
        with open("/etc/cyops-release", 'r') as flavor_details:
            for line in flavor_details:
                if re.search(r'product_type=', line):
                    return line.split("=")[1].strip() 
        
    def __init__(self):
        self.flavor = self.get_flavor()
        logging.debug("Flavor is " + self.flavor)
        self.data_file = "/opt/cyops/adm/data.json"
        self.all_choices = {}

    def merge(self, a, b):
        for key in b:
            if key in a:
                if isinstance(a[key], dict) and isinstance(b[key], dict):
                    self.merge(a[key], b[key])
                elif a[key] == b[key]:
                    pass # same leaf value
            else:
                logging.debug("Adding key " + key)
                a[key] = b[key]

    def get_choices_recursively(self, flavor, d):
        logging.debug(flavor)
        if not flavor in d.keys():
            return
        flavor_data = d[flavor]
        logging.debug(flavor_data)
        if 'isa' in flavor_data.keys():
            for relationship in flavor_data['isa']:
                self.get_choices_recursively(relationship, d)
                for key in flavor_data.keys():
                    if key != "isa":
                        #self.all_choices.update(dict(flavor_data[key]))
                        logging.debug("Updating key " + key + " for flavor " + flavor) 
                        self.merge(self.all_choices, {key : flavor_data[key]})
        else: 
            logging.debug("Updating all-choices for flavor " + flavor)     
            #self.all_choices.update(flavor_data)
            self.merge(self.all_choices, flavor_data)

    def get_names_recursively(self, flavor, d, names_list):
        if not flavor in d.keys():
            return
        flavor_data = d[flavor]
        if 'isa' in flavor_data.keys():
            names_list.extend(flavor_data['isa'])
            for relationship in flavor_data['isa']:
                self.get_names_recursively(relationship, d, names_list)

    def complete(self):
        with open(self.data_file) as json_file:
            all_data = json.load(json_file)
            self.get_choices_recursively(self.flavor, all_data)
            if len(sys.argv) > 1:
                i = 1
                while i < len(sys.argv):
                    if sys.argv[i] in self.all_choices.keys():
                        self.all_choices = self.all_choices[sys.argv[i]]
                    i = i + 1
            logging.debug(self.all_choices)
            print(' \n'.join(self.all_choices))

    def check_if_valid_option(self):
        if ('-h' in sys.argv) or ('--help' in sys.argv):
            return
        with open(self.data_file) as json_file:
            all_data = json.load(json_file) 
            self.get_choices_recursively(self.flavor, all_data)
            i = 1
            while i < len(sys.argv)-1:
                if sys.argv[i] not in self.all_choices.keys():
                    print("This " + sys.argv[i] + " subcommand is not applicable for " + self.flavor)
                    exit(1)
                self.all_choices = self.all_choices[sys.argv[i]]
                i = i + 1

    def print_all_names_for_current_flavors(self):
        all_names = [self.flavor]
        with open(self.data_file) as json_file:
            all_data = json.load(json_file) 
            self.get_names_recursively(self.flavor, all_data, all_names)
        print("\n".join(set(all_names)))
             
def start():
    loader = Loader()
    # There is special input flag "--check-if-valid" to the script with returns non-zero exit code if given option
    # is invalid for current flavor.
    if len(sys.argv) > 1 and sys.argv[-1] == "--check-if-valid":
        loader.check_if_valid_option()
        return
    if len(sys.argv) > 1 and sys.argv[-1] == "--get-all-names-for-current-flavor":
        loader.print_all_names_for_current_flavors()
        return
    logging.debug(sys.argv)
    loader.complete()

if __name__ == "__main__":
    start()
