#!/usr/bin/env python3
import argparse, sys, socket, json, os, subprocess, logging
from proxy_config import setHttpProxy, setNoProxy, setHttpsProxy, update_etc_yum_conf, get_full_proxy_url, update_etc_environment, list_proxy

if os.getuid() != 0:
    print("This script must be run as 'root'")
    exit(1)


def add_ipv6_subparser(subparsers):
    parser_ipv6 = subparsers.add_parser('ipv6', help='Handle IPv6 operations')
    parser_ipv6.set_defaults(operation='ipv6')
    parser_ipv6.add_argument('--enable', action='store_true', help='Enable IPv6 protocol. Requires system reboot.', dest='enable_ipv6')


def add_set_https_proxy_subparser(subparsers):
    parser_sethttpsproxy = subparsers.add_parser('set-https-proxy', help='Set proxy to be used for outbound https requests')
    parser_sethttpsproxy.set_defaults(operation='set-https-proxy')
    parser_sethttpsproxy.add_argument('--host', help='Proxy hostname', dest='proxy_hostname', required=True)
    parser_sethttpsproxy.add_argument('--port', help='Proxy port', dest='proxy_port', required=True, type=int)
    parser_sethttpsproxy.add_argument('--protocol', help='Proxy protocol', dest='proxy_protocol', required=True)
    parser_sethttpsproxy.add_argument('--user', help='Proxy username', dest='proxy_username', default=None, required='--password' in sys.argv)
    parser_sethttpsproxy.add_argument('--password', help='Proxy password', dest='proxy_password', default=None)


def add_set_http_proxy_subparser(subparsers):
    parser_sethttpproxy = subparsers.add_parser('set-http-proxy', help='Set proxy to be used for outbound http requests')
    parser_sethttpproxy.set_defaults(operation='set-http-proxy')
    parser_sethttpproxy.add_argument('--host', help='Proxy hostname', dest='proxy_hostname', required=True)
    parser_sethttpproxy.add_argument('--port', help='Proxy port', dest='proxy_port', required=True, type=int)
    parser_sethttpproxy.add_argument('--protocol', help='Proxy protocol', dest='proxy_protocol', required=True)
    parser_sethttpproxy.add_argument('--user', help='Proxy username', dest='proxy_username', default=None, required='--password' in sys.argv)
    parser_sethttpproxy.add_argument('--password', help='Proxy password', dest='proxy_password', default=None)


def add_set_no_proxy_subparser(subparsers):
    parser_setnoproxy = subparsers.add_parser('set-no-proxy', help='List of addresses not be routed through proxy')
    parser_setnoproxy.set_defaults(operation='set-no-proxy')
    parser_setnoproxy.add_argument('--host', help='Comma-separated list of hostnames', dest='proxy_hostname', required=True)


def add_list_proxy_subparser(subparsers):
    parser_list_proxy = subparsers.add_parser('list-proxy', help='List proxy details')
    parser_list_proxy.set_defaults(operation='list-proxy')


def define_parser(parser):
    visible_options = "{ipv6,set-https-proxy,set-http-proxy,set-no-proxy,list-proxy}"
    subparsers = parser.add_subparsers(metavar=visible_options)
    add_ipv6_subparser(subparsers)
    add_set_https_proxy_subparser(subparsers)
    add_set_http_proxy_subparser(subparsers)
    add_set_no_proxy_subparser(subparsers)
    add_list_proxy_subparser(subparsers)

    for name, subp in subparsers.choices.items():
        subp._positionals.title = 'subcommand options are'
        subp._optionals.title = 'subcommand options are'


class CustomHelpFormatter(argparse.HelpFormatter):

    def _format_action(self, action):
        if type(action) == argparse._SubParsersAction:
            # inject new class variable for subcommand formatting
            subactions = action._get_subactions()
            invocations = [self._format_action_invocation(a) for a in subactions]
            self._subcommand_max_length = max(len(i) for i in invocations)

        if type(action) == argparse._SubParsersAction._ChoicesPseudoAction:
            # format subcommand help line
            subcommand = self._format_action_invocation(action)  # type: str
            width = self._subcommand_max_length
            help_text = ""
            if action.help:
                help_text = self._expand_help(action)
            return "  {:{width}}    {}\n".format(subcommand, help_text, width=width)

        elif type(action) == argparse._SubParsersAction:
            # process subcommand help section
            msg = '\n'
            for subaction in action._get_subactions():
                msg += self._format_action(subaction)
            return msg
        else:
            return super(CustomHelpFormatter, self)._format_action(action)

def get_dash_string():

    return "------------------------------------------------"

def print_note(msg):

    dash_string = get_dash_string()
    note = dash_string + "\n"
    note = note + "Note:" + "\n"
    note = note + msg + "\n"
    note = note + dash_string + "\n"
    print (note)
    return

def print_reload_note():
    command_to_run = "'systemctl restart celeryd uwsgi'"
    note_msg = "Proxy Setting applied successfully. For the settings to take effect, you need to:" + "\n"
    note_msg = note_msg + "1. Restart services using the following command: " + command_to_run + "\n"
    note_msg = note_msg + "2. Logout and connect to the session again to load the latest environment."
    print_note(note_msg)

def start():
    parser = argparse.ArgumentParser(prog="csadm network", formatter_class=CustomHelpFormatter)
    parser._positionals.title = 'subcommand options are'
    parser._optionals.title = 'subcommand options are'

    define_parser(parser)

    args = parser.parse_args()

    if not 'operation' in dir(args):
        parser.print_help(sys.stderr)
        exit(1)

    if str(args.operation) == 'ipv6':
        if args.enable_ipv6:
            ret = os.system("/opt/cyops/configs/scripts/ipv6.sh --enable")
            if ret != 0:
                exit(1)
            return

    elif str(args.operation) == 'set-https-proxy':
        protocol = args.proxy_protocol
        host = args.proxy_hostname
        port = args.proxy_port
        username = args.proxy_username
        password = args.proxy_password

        proxy = protocol + "://" + host + ":" + str(port)
        update_etc_yum_conf(proxy, username, password)
        key = "export https_proxy"
        new_value = get_full_proxy_url(protocol, host, port, username, password)
        key = "https_proxy"
        # The users having to login shell will get proxy variables in env when we update /etc/environment. For example (sudo -u nginx <pip_command_here)
        update_etc_environment(key, new_value, append_to_existing_value=False)
        setHttpsProxy(host, port, username, password)
        print_reload_note()
        return

    elif str(args.operation) == 'set-http-proxy':

        protocol = args.proxy_protocol
        host = args.proxy_hostname
        port = args.proxy_port
        username = args.proxy_username
        password = args.proxy_password
       
        proxy = protocol + "://" + host + ":" + str(port)
        update_etc_yum_conf(proxy, username, password)
        key = "export http_proxy"
        new_value = get_full_proxy_url(protocol, host, port, username, password)
        key = "http_proxy"
        update_etc_environment(key, new_value, append_to_existing_value=False)
        setHttpProxy(host, port, username, password)
        print_reload_note()
        return 

    elif str(args.operation) == 'set-no-proxy':
        proxy = args.proxy_hostname
        key = "no_proxy"
        update_etc_environment(key, proxy, append_to_existing_value=False)
        setNoProxy(proxy)
        print_reload_note()
        return

    elif str(args.operation) == 'list-proxy':
        list_proxy()
        return

    # Execution should not come to this stage. If comes, then print help
    parser.print_help(sys.stderr)

if __name__ == '__main__':
    start()

exit(0)
