#!/bin/sh

set -eu

generate_and_copy_certs()
{
    local NEW_HOSTNAME=$1

    if ! grep -q "IP\.2" /opt/cyops/certs/leaf.openssl.conf; then 
        sed -i 's/\s*IP.1\s*=\s*127.0.0.1/IP.1 = ::1\nIP.2 = 127.0.0.1/g' /opt/cyops/certs/leaf.openssl.conf
    fi

    if ! grep -q "DNS\.2" /opt/cyops/certs/leaf.openssl.conf; then
        sed -i 's/\s*DNS\.1\s*=\s*cybersponse\.localhost/DNS.1 = cybersponse.localhost\nDNS.2 = localhost/g' /opt/cyops/certs/leaf.openssl.conf
    fi
 
    local CURRENT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
    local COUNTRY='US'
    local STATE='Virginia'
    local LOCATION='Arlington'
    local ORGANIZATION='CyberSponse Inc'
    local TEAM='Support'
    local ROOT_CERT=/etc/pki/ca-trust/source/anchors/cs.loc.root.crt
    local ROOT_KEY=/etc/pki/cyops/cs.loc.root.key
    local ROOT_LEAF_CONF=/opt/cyops/certs/leaf.openssl.conf
    local CERT_DIR=/etc/pki/cyops
    local NGINX_CERT_DIR=/etc/nginx/ssl/

    #hostnamectl set-hostname $new_hostname
    local TEMP_CERT_DIR="${CURRENT_DIR}/tmp"

    mkdir -p $TEMP_CERT_DIR
    cd $TEMP_CERT_DIR

    # Generate Certificate Signing Request
    openssl req -nodes -batch -new -newkey rsa:2048 \
                -subj "/C=$COUNTRY/ST=$STATE/L=$LOCATION/O=$ORGANIZATION/OU=$TEAM/CN=${NEW_HOSTNAME}" \
                -sha256 -days 3650 -keyout server.leaf.key \
                -out server.leaf.csr -config ${ROOT_LEAF_CONF} \
                -reqexts v3_req -passin pass: 

    # Generate certificates
    openssl x509 -req -in server.leaf.csr -extfile ${ROOT_LEAF_CONF} \
                 -extensions v3_leaf -sha256 -days 3650 -CA ${ROOT_CERT} \
                 -CAkey ${ROOT_KEY} -CAcreateserial \
                 -out server.leaf.crt 

    echo "Generating pem file from crt and key"
    cat server.leaf.crt server.leaf.key > server.leaf.pem

    # Copy certificates to /etc/pki/cyops
    rm -rf ${CERT_DIR}/server.leaf.*
    cp server.leaf.* ${CERT_DIR}
    chmod 644 ${CERT_DIR}/*

    # Add new certificates to ca-trust
    update-ca-trust force-enable
    update-ca-trust extract

    # Update nginx certificates
    rm -rf ${NGINX_CERT_DIR}/server.leaf.*
    cp server.leaf.* ${NGINX_CERT_DIR}
    chmod 644 $NGINX_CERT_DIR/server.leaf.*
    chown -R root:root ${NGINX_CERT_DIR}

    # update the certificate in auth
    rm -f /opt/cyops-auth/ssl/server.leaf.*
    cp server.leaf.* /opt/cyops-auth/ssl/
    chmod 644 /opt/cyops-auth/ssl/server.leaf.*
    chown nginx:nginx /opt/cyops-auth/ssl/server.leaf.*

    if [ "$TEMP_CERT_DIR" != "/" ]; then
        cd ..
        rm -rf $TEMP_CERT_DIR
    fi
}

check_root()
{
    if [ $EUID -ne 0 ]; then
        echo "You need to be root to perform this command."
        exit 1
    fi
}

enable_ipv6_in_sysctl()
{
    local keys=("net.ipv6.conf.all.disable_ipv6
                net.ipv6.conf.default.disable_ipv6
                net.ipv6.conf.lo.disable_ipv6
                net.ipv6.conf.all.accept_redirects
                net.ipv6.conf.default.accept_redirects
                net.ipv6.route.flush
                ")
    local file=/etc/sysctl.conf
    for key in $keys
    do
        grep  -q "^\s*$key" /etc/sysctl.conf || echo "$key = 0" >> $file
        sed -i "s/$key =.*$/$key = 0/" $file
    done
}

clear_api_cache()
{
    rm -rf /opt/cyops-api/app/cache/prod
    restorecon -R /opt/cyops-api
    systemctl restart redis
    systemctl restart nginx
    sudo -u nginx php /opt/cyops-api/app/console --env=prod >/dev/null
    sudo -u nginx php /opt/cyops-api/app/console cache:clear --env=prod --no-interaction
}

process_confirmation()
{ 
    local in_msg=$1
    local exit_on_non_aggreement=$2
    local confirmation_msg="Do you want to continue? [yes/no]:"
    local input
    local correct_input=0
    local dash_str=`get_dash_line`
    local error_msg="\n${dash_str}\nError:\n"
          error_msg="${error_msg}Entered value is invalid\n"
          error_msg="${error_msg}${dash_str}"

    while [ "X$correct_input" = "X0" ]
    do
        echo -e $in_msg
        read -p "$confirmation_msg" input
        if echo "$input" | grep -iE "^yes$|^no$|^y$|^n$" >/dev/null ;then
            correct_input=1
        else
            echo -e $error_msg
        fi
    done

    if [ "X$exit_on_non_aggreement" = "X1" ] ;then
        if echo "$input" | grep -iE "^no$|^n$" >/dev/null ;then
            echo "Exiting..."
            exit 0
        fi
    fi
}

get_dash_line()
{
    echo "-------------------------------------------------"
}

take_confirmation_for_ipv6_enablement()
{
    local dash_str=`get_dash_line`
    local note_msg="${dash_str}\nNote:\n"
          note_msg="${note_msg}Running this command will enable IPv6 protocol on your system, which requires\n"
          note_msg="${note_msg}restarting of the network and product services.\n"
          note_msg="${note_msg}${dash_str}"
    process_confirmation "$note_msg" 1
}

reboot_the_system()
{
    local dash_str=`get_dash_line`
    local note_msg="${dash_str}\nNote:\n"
          note_msg="${note_msg}You need to re-run command 'csadm network ipv6 --enable'\n"
          note_msg="${note_msg}after system reboot to complete IPv6 configuration.\n"
          note_msg="${note_msg}${dash_str}"
    echo -e $note_msg
    read -p "Press enter/return to reboot..." opt
    reboot
}

modify_ifcfg()
{
    local interface_name=`ip link | awk -F: '$0 !~ "lo|vir|wl|^[^0-9]"{print $2;getline}' | awk '{$1=$1};1'`
    local ifcfg_file_path="/etc/sysconfig/network-scripts/ifcfg-${interface_name}"
    if [ ! -f $ifcfg_file_path ]; then
        return
    fi
    echo "IPV6INIT=yes" >> $ifcfg_file_path
    echo "DHCPV6C=yes" >> $ifcfg_file_path 
}
modify_rc_local()
{
    rc_local=/etc/rc.local
    etc_issue=/etc/issue
    etc_issue_net=/etc/issue.net
    # First check whether IPV6 entry is there in rc.local. If NOT, then add it.
    if ! grep -qi "ipv6" $rc_local; then
        # In below command, we are finding ipv6 excluding ::1 and fe80 (link local ipv6)
        # Why we are sleeping for 10 secs: This is required as system need some time to actually get the IPV6 from publisher.
        # Please note that number 10 is try and error basis number got from QA.
        echo "sleep 10" >> $rc_local
        echo "IPV6=\$(ip -6 addr | grep inet6 | awk -F '[ \t]+|/' '{print \$3}' | grep -v ^::1 | grep -v ^fe80)" >> $rc_local
        echo "echo \"IPv6: \$IPV6\" | tee --append $etc_issue $etc_issue_net" >> $rc_local
    fi
}
enable_ipv6()
{
    take_confirmation_for_ipv6_enablement
    local reboot_required=0
    if grep -q "ipv6.disable=1" /etc/default/grub; then
       #ipv6 is disabled in grub
       sed -i 's/ipv6.disable=1/ipv6.disable=0/g' /etc/default/grub
       grub2-mkconfig -o /boot/grub2/grub.cfg
       reboot_required=1
    fi
    enable_ipv6_in_sysctl
    modify_ifcfg
    modify_rc_local
    if [ "X$reboot_required" = "X1" ]; then
        reboot_the_system
    else
        systemctl restart network   
    fi

    #Update nginx configuration files
    /opt/cyops/configs/scripts/update_nginx_listen_config.sh

    local cyops_conf_files=("/opt/cyops-api/app/config/parameters_prod.yml  
                       /opt/cyops-workflow/sealab/sealab/config.ini 
                       /opt/cyops-integrations/integrations/configs/config.ini 
                       /opt/cyops/configs/database/db_config.yml 
                       /opt/cyops/configs/cyops-api/postman.yml 
                       /etc/cyops/config.yml 
                       /opt/cyops/configs/cyops-api/parameter_change.yml 
                       /opt/cyops-auth/utilities/das.ini 
                       /etc/nginx/conf.d/cyops*.conf
                       /opt/cyops-routing-agent/postman/config.ini 
                       /etc/elasticsearch/elasticsearch.yml")

    for conf in $cyops_conf_files
    do
        echo "Updating $conf"
        sed -i 's/127.0.0.1/localhost/g' $conf
    done

    sed -i 's/^\s*bind 127.0.0.1/bind ::1 localhost/g' /etc/redis.conf
    local curr_hostname=`hostname`
    generate_and_copy_certs $curr_hostname
    clear_api_cache
    csadm services --restart
}

print_help()
{
   local help_type=$1
   local help_one_liner="IPv6 operations"
   if [ "detailed" = "$help_type" ]; then
       echo "$help_one_liner"
       echo "usage: csadm ipv6 [options]"
       echo ""
       echo "subcommand options are:"
       echo "    --enable                    Enable IPv6 protocol. Requires system reboot."
       echo "    --help                      Show this text"
   elif [ "short" = "$help_type" ];then
       echo "$help_one_liner"
   fi
   exit 0
}

##################################
# MAIN/main/Main starts here
##################################
check_root
# read the input options
if [ $# -ne 1 ]; then
    print_help 'detailed'
fi
case "$1" in
    --enable)
        enable_ipv6
        break;;
    *)
        print_help 'detailed'
        break;;
esac
exit 0
